library(LaplacesDemon)

# Parameters
n <- 200  # sample size before selection effects
var.true <- 9 # intrinsic standard deviation of absolute magnitudes

# sample the redshifts
z.pts <- 1:100/100
z.prob <- (1+z.pts)^2
z.sim <- sample(z.pts,size=n,replace=TRUE,prob=z.prob)

# read in Lambda CDM model
LCDM <- read.table("LambdaCDM.txt", header=TRUE)

# Simulate absolute magnitude
M.sim <- rnorm(n, -19.3, sqrt(var.true))
# Compute the apparent magnitudes, as absolute magnitude + mu (i.e., distance modulus)
m.sim <- M.sim + LCDM[round(z.sim*100),2]

# Select if m.sim < 24
z.sel <- z.sim[m.sim<24]
M.sel <- M.sim[m.sim<24]
m.sel <- m.sim[m.sim<24]

# observed sample size
N <- length(m.sel)


#### Question 1 ####

# code to simulate data given in Table 1

z.miss <- z.sim[m.sim>=24]
M.miss <- M.sim[m.sim>=24]
m.miss <- m.sim[m.sim>=24]

M.cutoff <- 24 - LCDM[round(z.pts*100),2]

plot(z.sel,M.sel,col="blue",pch=19,xlab="z (redshift)",ylab="M (asbsolute magnitude)", ylim=c(-30,-5),xlim=c(0,1))
points(z.miss,M.miss,col="red",pch=19)
lines(z.pts,M.cutoff,col="black",lwd=2)
legend("topright",c("cuts","observed","missed"),lty=c(1,NA,NA),lwd=c(2,NA,NA),pch=c(NA,19,19),col=c("black","blue","red"))


#### Question 2 ####

mu <- NULL; sig2 <- NULL
mu[1] <- -19.3; sig2[1] <- 9

# set some priors
#mu0 <- 0; tau2 <- Inf; nu = 0; beta2 = 0

# set some priors
mu0 <- -19.3; tau2 <- 20^2; nu = 0.02; beta2 = 0.02

# Gibbs sampler
num.draws <- 12000
for(i in 2:num.draws){
	
	# Step 1: update mu from its conditional posterior dist'n given sig2
	a = (sum(M.sel)/sig2[i-1] + mu0/tau2)/(N/sig2[i-1] + 1/tau2)
	b = 1/(N/sig2[i-1] + 1/tau2)
	mu[i] <- rnorm(1,mean=a,sd=sqrt(b))
	
	# Step 2: update sig2 from its conditional posterior dist'n given mu
	sig2[i] <- sum(((M.sel-mu[i])^2)+beta2)/rchisq(1,df=(N+nu))	
}

p2.mu <- mu
p2.sig2 <- sig2

par(mfrow=c(2,1))
plot(p2.mu,type="l")
plot(p2.sig2,type="l")

par(mfrow=c(1,2))
hist(p2.mu[2001:12000],xlim=c(-22,-14), main="",xlab=expression(mu),prob=TRUE)
abline(v=-19.3,col="red",lwd=2)
hist(log(p2.sig2[2001:12000]),xlim=c(1,3), main="",xlab=expression(log(sigma^2)),prob=TRUE)
abline(v=log(9),col="red",lwd=2)


#### Question 3 ####

dtnorm <- function(x, mean, sd, truncation, log=FALSE){
	
	value <- 0
	if( x <= truncation){
		value <- dnorm(x,mean,sd)/pnorm(truncation,mean,sd)
	}
	
	if(log == TRUE) value <- log(value)
	
	value
}


#### Question 4 ####

Mod2Gibbs <- function(Mags,z,start.vals,draw.num=10000,jump.par){
    accept.mu = 0
    accept.sig2 = 0
    
    Draws = matrix(NA,draw.num,2)
	
    #priors (can change)
    mu0 <- -19.3
    tau2 <- 20^2
    beta2 <- 0.02
    nu <- 0.02 
	
    logPost = function(mu,sig2){
        logLik <- 0
        for(i in 1:length(Mags)){
        		logLik <- logLik + dtnorm(x=Mags[i],mean=mu,sd=sqrt(sig2),truncation=(24-LCDM[round(z[i]*100),2]),log=TRUE)
        }
        #sum(dtnorm(x=Mags,mean=mu,sd=sqrt(sig2),truncation=LCDM[round(z*100),2]))
        logPrior <- dnorm(mu,mu0,sqrt(tau2),log=TRUE) + log(beta2) + dinvchisq(sig2,nu,log=TRUE)
        return(logLik + logPrior)
    }
		
	Draws[1,] <- start.vals
	
	for(i in 2:draw.num){
		
		#update mu
		mu.star <- rnorm(1,Draws[i-1,1],jump.par[1])
		log.ratio <- logPost(mu.star,Draws[i-1,2]) - logPost(Draws[i-1,1],Draws[i-1,2])
		ratio = exp( min(log.ratio,100) )
		temp = runif(1)
		if(temp < min(ratio,1)){
			Draws[i,1] <- mu.star
			accept.mu = accept.mu + 1
		}else{
			Draws[i,1] <- Draws[i-1,1]
		}
		
		#update sig2
		sig2.star <- rlnorm(1,log(Draws[i-1,2]),jump.par[2])
		log.ratio <- logPost(Draws[i,1],sig2.star)- 
							dlnorm(sig2.star,log(Draws[i-1,2]),jump.par[2],log=TRUE)-
							logPost(Draws[i,1],Draws[i-1,2])+
							dlnorm(Draws[i-1,2],log(sig2.star),jump.par[2],log=TRUE); 
		ratio = exp( min(log.ratio,100) )
		temp = runif(1)
		if(temp < min(ratio,1)){
			Draws[i,2] <- sig2.star
			accept.sig2 = accept.sig2 + 1
		}else{
			Draws[i,2] <- Draws[i-1,2]
		}
			
	}
	output <- list(Draws,accept.mu/draw.num,accept.sig2/draw.num)
	names(output) <- c("Draws","accept.ratio.mu","accept.ratio.sig2")
	return(output)
}

p4 <- Mod2Gibbs(M.sel,z.sel,start.vals=c(-19.3,9),draw.num=11000,jump.par=c(0.82^2,0.62^2))

par(mfrow=c(2,1))
plot(p4$Draws[,1],type="l")
plot(p4$Draws[,2],type="l")

par(mfrow=c(1,2))
hist(p4$Draws[1001:11000,1],xlim=c(-22,-14), main="",xlab=expression(mu),prob=TRUE)
abline(v=-19.3,col="red",lwd=2)
hist(log(p4$Draws[1001:11000,2]),xlim=c(1,3), main="",xlab=expression(log(sigma^2)),prob=TRUE)
abline(v=log(9),col="red",lwd=2)



#### Question 5 ####

mu <- NULL; sig2 <- NULL
mu[1] <- -19.3; sig2[1] <- 9

# set some priors
#mu0 <- 0; tau2 <- Inf; nu = 0; beta2 = 0

# set some priors
mu0 <- -19.3; tau2 <- 20^2; nu = 0.02; beta2 = 0.02

# Gibbs sampler
num.draws <- 12000
for(i in 2:num.draws){
	
	# Step 1: update mu from its conditional posterior dist'n given sig2
	a = (sum(M.sim)/sig2[i-1] + mu0/tau2)/(n/sig2[i-1] + 1/tau2)
	b = 1/(n/sig2[i-1] + 1/tau2)
	mu[i] <- rnorm(1,mean=a,sd=sqrt(b))
	
	# Step 2: update sig2 from its conditional posterior dist'n given mu
	sig2[i] <- sum(((M.sim-mu[i])^2)+beta2)/rchisq(1,df=(n+nu))
					# inverse-chi-squared dist'n?
	
}

p5.mu <- mu
p5.sig2 <- sig2

par(mfrow=c(2,1))
plot(p5.mu,type="l")
plot(p5.sig2,type="l")

par(mfrow=c(1,2))
hist(p5.mu[2001:12000],xlim=c(-22,-14),xlab=expression(mu),prob=TRUE, main="")
abline(v=-19.3,col="red",lwd=2)
hist(log(p5.sig2[2001:12000]),xlim=c(1,3),xlab=expression(log(sigma^2)),prob=TRUE, main="")
abline(v=log(9),col="red",lwd=2)


# > var(p5.mu)
# [1] 0.0406019
# > var(p5.sig2)
# [1] 0.6823052
# > var(p4$Draws[,1])
# [1] 0.2438744
# > var(p4$Draws[,2])
# [1] 3.120121

### Variances larger for question 4 than question 5


#### Problem 6

# Parameters
n.new <- 2000  # sample size before selection effects
var.true <- 9 # intrinsic standard deviation of absolute magnitudes

# sample the redshifts
z.pts <- 1:100/100
z.prob <- (1+z.pts)^2
z.sim.new <- sample(z.pts,size=n.new,replace=TRUE,prob=z.prob)

# read in Lambda CDM model
LCDM <- read.table("LambdaCDM.txt", header=TRUE)

# Simulate absolute magnitude
M.sim.new <- rnorm(n.new, -19.3, sqrt(var.true))
# Compute the apparent magnitudes, as absolute magnitude + mu (i.e., distance modulus)
m.sim.new <- M.sim.new + LCDM[round(z.sim.new*100),2]

# Select if m.sim < 24
z.sel.new <- z.sim.new[m.sim.new<24]
M.sel.new <- M.sim.new[m.sim.new<24]
m.sel.new <- m.sim.new[m.sim.new<24]

# observed sample size
N.new <- length(m.sel.new)

p6 <- Mod2Gibbs(M.sel.new,z.sel.new,start.vals=c(-19.3,9),draw.num=11000,jump.par=c(0.82^2,0.62^2))

par(mfrow=c(2,1))
plot(p6$Draws[,1],type="l")
plot(p6$Draws[,2],type="l")

par(mfrow=c(1,2))
hist(p6$Draws[1001:11000,1],xlim=c(-22,-14), main="",xlab=expression(mu),prob=TRUE)
abline(v=-19.3,col="red",lwd=2)
hist(log(p6$Draws[1001:11000,2]),xlim=c(1,3), main="",xlab=expression(log(sigma^2)),prob=TRUE)
abline(v=log(9),col="red",lwd=2)

var(p6$Draws[,1])
#0.05645154
# > N.new
# [1] 890


########## Question 7 ###########

AppMagGibbs <- function(Mags,z,start.vals,draw.num=10000,jump.par){
    accept.mu = 0
    accept.sig2 = 0
    
    Draws = matrix(NA,draw.num,2)
	
    #priors (can change)
    mu0 <- -19.3 
    tau2 <- 20^2
    beta2 <- 0.02
    nu <- 0.02 
	
    logPost = function(mu,sig2){
        logLik <- 0
        for(i in 1:length(Mags)){
        		logLik <- logLik + dtnorm(x=Mags[i],mean=mu+LCDM[round(z[i]*100),2], sd=sqrt(sig2),truncation=24,log=TRUE)
        }
        #sum(dtnorm(x=Mags,mean=mu,sd=sqrt(sig2),truncation=LCDM[round(z*100),2]))
        logPrior <- dnorm(mu,mu0,sqrt(tau2),log=TRUE) + log(beta2) + dinvchisq(sig2,nu,log=TRUE)
        return(logLik + logPrior)
    }
		
	Draws[1,] <- start.vals
	
	for(i in 2:draw.num){
		
		#update mu
		mu.star <- rnorm(1,Draws[i-1,1],jump.par[1])
		log.ratio <- logPost(mu.star,Draws[i-1,2]) - logPost(Draws[i-1,1],Draws[i-1,2])
		ratio = exp( min(log.ratio,100) )
		temp = runif(1)
		if(temp < min(ratio,1)){
			Draws[i,1] <- mu.star
			accept.mu = accept.mu + 1
		}else{
			Draws[i,1] <- Draws[i-1,1]
		}
		
		#update sig2
		sig2.star <- rlnorm(1,log(Draws[i-1,2]),jump.par[2])
		log.ratio <- logPost(Draws[i,1],sig2.star)- 
							dlnorm(sig2.star,log(Draws[i-1,2]),jump.par[2],log=TRUE)-
							logPost(Draws[i,1],Draws[i-1,2])+
							dlnorm(Draws[i-1,2],log(sig2.star),jump.par[2],log=TRUE); 
		ratio = exp( min(log.ratio,100) )
		temp = runif(1)
		if(temp < min(ratio,1)){
			Draws[i,2] <- sig2.star
			accept.sig2 = accept.sig2 + 1
		}else{
			Draws[i,2] <- Draws[i-1,2]
		}
		#print(i)	
	}
	output <- list(Draws,accept.mu/draw.num,accept.sig2/draw.num)
	names(output) <- c("Draws","accept.ratio.mu","accept.ratio.sig2")
	return(output)
}

p7 <- AppMagGibbs(m.sel,z.sel,start.vals=c(-19.3,9),draw.num=11000,jump.par=c(0.9^2,0.65^2))

par(mfrow=c(2,1))
plot(p7$Draws[,1],type="l")
plot(p7$Draws[,2],type="l")

par(mfrow=c(1,2))
hist(p7$Draws[1001:11000,1],xlim=c(-22,-14), main="",xlab=expression(mu),prob=TRUE)
abline(v=-19.3,col="red",lwd=2)
hist(log(p7$Draws[1001:11000,2]),xlim=c(1,3), main="",xlab=expression(log(sigma^2)),prob=TRUE)
abline(v=log(9),col="red",lwd=2)
